"""xinput window"""
# -*- encoding: utf-8 -*-
#
# This file is free software by d0n <d0n@janeiskla.de>
#
# You can redistribute it and/or modify it under the terms of the GNU -
# Lesser General Public License as published by the Free Software Foundation
#
# This is distributed in the hope that it will be useful somehow.
#
# !WITHOUT ANY WARRANTY!
#
# Without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.

try:
	from tkinter import StringVar, Button, Entry, Frame, Label, Tk
except ImportError:
	from Tkinter import StringVar, Button, Entry, Frame, Label, Tk

try:
	import readline
except ImportError:
	pass

class XInput(Frame):
	"""password clipping class for tkinter.Frame"""
	inp = None
	def __init__(
          self, master, message, exchange='', noop=None, noin=None):
		self.noop = noop
		self.noin = noin
		self.exchg = exchange
		self.message = message
		Frame.__init__(self, master)
		self.pack()
		self.inputwindow()
	def _enterexit(self, _=None):
		"""exit by saving challenge-response for input"""
		self.inp = True if not self.input else self.input.get()
		self.quit()
	def _exit(self, _=None):
		"""just exit (for ESC mainly)"""
		self.quit()
	def inputwindow(self):
		"""password input window creator"""
		self.lbl = Label(self, text=self.message)
		self.lbl.pack()
		okside = {}
		self.ok = Button(self)
		self.ok.bind("<Control-c>", self._exit)
		self.ok.bind("<Escape>", self._exit)
		self.ok.bind("<Return>", self._enterexit)
		self.input = False
		self.ok["command"] = self._enterexit
		if not self.noin:
			self.entry = Entry(self, show=self.exchg)
			self.entry.bind("<Return>", self._enterexit)
			self.entry.bind("<Control-c>", self._exit)
			self.entry.bind("<Escape>", self._exit)
			self.entry.focus_set()
			self.entry.pack()
			self.input = StringVar()
			self.entry["textvariable"] = self.input
			okside = {'side': 'left'}
			okaction = self._enterexit
		if not self.noop:
			self.cl = Button(self)
			self.cl.bind("<Escape>", self._exit)
			self.cl["text"] = "cancel"
			self.cl["command"] = self._exit
			self.cl.pack(side="right")
			self.ok["command"] = self._exit
		self.ok["text"] = "ok"
		if self.noin:
			self.ok.focus_set()
		self.ok.pack(okside)

def xinput(message='enter input'):
	"""x screen input window"""
	root = Tk()
	try:
		pwc = XInput(root, message)
	except KeyboardInterrupt:
		root.destroy()
	pwc.lift()
	pwc.mainloop()
	root.destroy()
	return pwc.inp

def xgetpass(message="input will not be displayed"):
	"""gui representing function"""
	root = Tk()
	try:
		pwc = XInput(root, message, exchange='*')
	except KeyboardInterrupt:
		root.destroy()
	pwc.lift()
	pwc.mainloop()
	root.destroy()
	return pwc.inp

def xmsgok(message='press ok to continue'):
	"""gui representing function"""
	root = Tk()
	try:
		pwc = XInput(root, message, noop=True, noin=True)
	except KeyboardInterrupt:
		root.destroy()
	pwc.lift()
	pwc.mainloop()
	root.destroy()

def xyesno(message='press ok to continue'):
	"""gui representing function"""
	# instanciate Tk and create window
	root = Tk()
	try:
		pwc = XInput(root, message, noin=True)
	except KeyboardInterrupt:
		root.destroy()
	pwc.lift()
	pwc.mainloop()
	root.destroy()
	return pwc.inp


if __name__ == '__main__':
	exit(1)



"""randomisation functions"""
from re import search
from os import getrandom

def random(limit=10, regex=r'[\w -~]'):
	"""random character by limit"""
	rets = []
	while True:
		try:
			out = getrandom(1).decode()
		except UnicodeDecodeError:
			continue
		try:
			if search(regex, out).group(0):
				rets.append(out.strip())
		except AttributeError:
			continue
		if len(rets) >= int(limit):
			break
	return ''.join(r for r in rets)

def biggerrand(num):
	"""greater random number"""
	while True:
		try:
			g = int(random(int(len(str(num))+1), regex=r'[0-9]*'))
		except ValueError:
			continue
		if g > int(num):
			break
	return g

def lowerrand(num):
	"""lower random number"""
	while True:
		try:
			g = int(random(int(len(str(num))), regex=r'[0-9]*'))
		except ValueError:
			continue
		if g > 1 and g < int(num):
			break
	return g

def randin(top, low=0):
	"""number in between low and top"""
	if low:
		low, top = top, low
	while True:
		try:
			g = int(random(int(len(str(top))), regex=r'[0-9]*'))
		except ValueError:
			continue
		if g > int(low) and g < int(top):
			break
	return g



"""x notification function"""
# -*- encoding: utf-8 -*-
#
# This file is free software by d0n <d0n@janeiskla.de>
#
# You can redistribute it and/or modify it under the terms of the GNU -
# Lesser General Public License as published by the Free Software Foundation
#
# This is distributed in the hope that it will be useful somehow.
#
# !WITHOUT ANY WARRANTY!
#
# Without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
from inspect import stack
try:
	import gi
	gi.require_version('Notify', '0.7')
	from gi.repository import Notify as xnote
except (AttributeError, ImportError) as err:
	def xnotify(*_): """xnotify faker function""" ;return
else:
	def xnotify(msg, name=stack()[1][3], wait=5):
		"""
		disply x notification usually in the upper right corner of the display
		"""
		try:
			xnote.init(str(name))
			note = xnote.Notification.new(msg)
			note.set_timeout(int(int(wait)*600))
			note.show()
		except (NameError, RuntimeError):
			pass



#!/usr/bin/env python3
# -*- encoding: utf-8 -*-
"""passcrypt module"""

from os import path, remove, environ, chmod, stat, makedirs

from time import time

from yaml import load, dump

from paramiko.ssh_exception import SSHException

from colortext import bgre, tabd, error

from system import userfind, filerotate, setfiletime

from net.ssh import SecureSHell

from secrecy.gpgtools import GPGTool, GPGSMTool

class PassCrypt(object):
	"""passcrypt main class"""
	dbg = None
	aal = None
	fsy = None
	sho = None
	gsm = None
	try:
		user = userfind()
		home = userfind(user, 'home')
	except FileNotFoundError:
		user = environ['USERNAME']
		home = path.join(environ['HOMEDRIVE'], environ['HOMEPATH'])
	user = user if user else 'root'
	home = home if home else '/root'
	plain = path.join(home, '.pwd.yaml')
	crypt = path.join(home, '.passcrypt')
	remote = ''
	reuser = user
	recvs = []
	sslcrt = ''
	sslkey = ''
	__weaks = {}
	__oldweaks = {}
	def __init__(self, *args, **kwargs):
		"""passcrypt init function"""
		for arg in args:
			if hasattr(self, arg):
				setattr(self, arg, True)
		for (key, val) in kwargs.items():
			if hasattr(self, key):
				setattr(self, key, val)
		if self.dbg:
			print(bgre(PassCrypt.__mro__))
			print(bgre(tabd(PassCrypt.__dict__, 2)))
			print(' ', bgre(self.__init__))
			print(bgre(tabd(self.__dict__, 4)))
		recvs = self.recvs
		if not self.recvs:
			if 'GPGKEYS' in environ.keys():
				recvs = environ['GPGKEYS'].split(' ')
			elif 'GPGKEY' in environ.keys():
				recvs = [environ['GPGKEY']]
		now = int(time())
		cache = '%s/.cache'%self.home
		timefile = '%s/pwclip.time'%cache
		if not path.exists(cache):
			makedirs(cache)
		age = None
		if not path.exists(timefile):
			with open(timefile, 'w+') as tfh:
				tfh.write(str(now))
			age = 14401
		self.age = int(now-int(stat(timefile).st_mtime)) if not age else age
		gsks = GPGSMTool().keylist(True)
		if self.gsm or (
              self.gsm is None and self.recvs and [
                  r for r in self.recvs if r in gsks]):
			self.gpg = GPGSMTool(*args, **kwargs)
		else:
			self.gpg = GPGTool(*args, **kwargs)
		self.ssh = SecureSHell(*args, **kwargs)
		write = False
		if self.remote and self._copynews_():
			write = True
		__weaks = self._readcrypt()
		self.__oldweaks = __weaks
		try:
			with open(self.plain, 'r') as pfh:
				__newweaks = load(pfh.read())
			#if not self.dbg:
			#	remove(self.plain)
			write = True
		except FileNotFoundError:
			__newweaks = {}
		if __newweaks:
			__weaks = __weaks if __weaks else {}
			for (su, ups) in __newweaks.items():
				for (usr, pwdcom) in ups.items():
					if su not in __weaks.keys():
						__weaks[su] = {}
					__weaks[su][usr] = pwdcom
		self.__weaks = __weaks
		if write:
			self._writecrypt(__weaks)

	def _copynews_(self):
		"""copy new file method"""
		if self.dbg:
			print(bgre(self._copynews_))
		if int(self.age) > 14400 and self.remote:
			try:
				return self.ssh.scpcompstats(
                    self.crypt, path.basename(self.crypt),
                    remote=self.remote, reuser=self.reuser)
			except FileNotFoundError:
				pass
			except SSHException as err:
				error(err)
		return False

	def _chkcrypt(self, __weak):
		"""crypt file checker method"""
		if self.dbg:
			print(bgre(self._chkcrypt))
		if self.__oldweaks == __weak:
			return True
		return False

	def _findentry(self, pattern, __weak):
		"""entry finder method"""
		if self.dbg:
			print(bgre(tabd({self._findentry: {'pattern': pattern}})))
		for (u, p) in __weak.items():
			if pattern == u or (
                  len(p) == 2 and len(pattern) > 1 and pattern in p[1]):
				return p
		return False

	def _readcrypt(self):
		"""read crypt file method"""
		if self.dbg:
			print(bgre(self._readcrypt))
		try:
			with open(self.crypt, 'r') as vlt:
				crypt = vlt.read()
		except FileNotFoundError:
			return False
		return load(str(self.gpg.decrypt(crypt)))

	def _writecrypt(self, __weak):
		"""crypt file writing method"""
		if self.dbg:
			print(bgre(self._writecrypt))
		kwargs = {'output': self.crypt}
		if self.recvs:
			kwargs['recipients'] = self.recvs
		filerotate(self.crypt, 2)
		for _ in range(0, 2):
			self.gpg.encrypt(message=dump(__weak), **kwargs)
			if self._chkcrypt(__weak):
				chmod(self.crypt, 0o600)
				now = int(time())
				setfiletime(self.crypt, (now, now))
				if self.remote:
					self._copynews_()
				return True
		return True

	def adpw(self, usr, pwd=None, com=None):
		"""password adding method"""
		if self.dbg:
			print(bgre(tabd({
                self.adpw: {'user': self.user, 'entry': usr,
                            'pwd': pwd, 'comment': com}})))
		pwdcom = [pwd if pwd else self.gpg.passwd()]
		if not pwd and not com:
			com = input('enter a comment: ')
		if com:
			pwdcom.append(com)
		if self.dbg:
			print('%s\n adduser = %s addpass = %s'%(
                self.adpw, usr, pwd))
		__weak = self._readcrypt()
		if not self.aal:
			__weak[self.user][usr] = pwdcom
		else:
			for u in __weak.keys():
				__weak[u][usr] = pwdcom
		self._writecrypt(__weak)
		return __weak

	def chpw(self, usr, pwd=None, com=None):
		"""change existing password method"""
		if self.dbg:
			print(bgre(tabd({
                self.chpw: {'user': self.user, 'entry': usr, 'pwd': pwd}})))
		pwdcom = [pwd if pwd else self.gpg.passwd()]
		if com:
			pwdcom.append(com)
		if not com and not pwd:
			com = input('enter a comment: ')
		if com:
			pwdcom.append(com)
		__weak = self._readcrypt()
		if __weak and self.user in __weak.keys() and \
              usr in __weak[self.user].keys():
			__weak[self.user][usr] = pwdcom
			self._writecrypt(__weak)
		return __weak

	def rmpw(self, usr):
		"""remove password method"""
		if self.dbg:
			print(bgre(tabd({self.rmpw: {'user': self.user, 'entry': usr}})))
		__weak = self._readcrypt()
		if not self.all:
			try:
				del __weak[self.user][usr]
			except KeyError:
				error('entry', usr, 'could not be found for', self.user)
		else:
			for u in __weak.keys():
				try:
					del __weak[u][usr]
				except KeyError:
					error('entry', usr, 'could not be found for', self.user)
			
		if __weak and self.user in __weak.keys() and \
              usr in __weak[self.user].keys():
			del __weak[self.user][usr]
			if self._writecrypt(__weak):
				if self.remote:
					self._copynews_()
		return __weak

	def lspw(self, usr=None, aal=None):
		"""password listing method"""
		if self.dbg:
			print(bgre(tabd({self.lspw: {'user': self.user, 'entry': usr}})))
		aal = True if aal else self.aal
		__ents = {}
		if self.__weaks:
			if aal:
				__ents = self.__weaks
				if usr:
					for (user, _) in self.__weaks.items():
						__match = self._findentry(usr, self.__weaks[user])
						if __match:
							__ents = {usr: __match}
							break
			elif self.user in self.__weaks.keys():
				__ents = self.__weaks[self.user]
				if usr:
					__match = self._findentry(usr, __ents)
					if __match:
						__ents = {usr: __match}
		return __ents

def lscrypt(usr, dbg=None):
	"""passlist wrapper function"""
	if dbg:
		print(bgre(lscrypt))
	__ents = {}
	if usr:
		__ents = PassCrypt().lspw(usr)
	return __ents




if __name__ == '__main__':
	exit(1)



#!/usr/bin/python3
"""command module of executor"""
from os import access, environ, getuid, getresuid, X_OK, name as osname
from os.path import abspath, join as pjoin

from sys import stdout as _stdout, stderr as _stderr

from subprocess import run, Popen, PIPE
# for legacy subprocess compatibility while DEVNULL is new in subprocess
try:
	from subprocess import DEVNULL
except ImportError:
	DEVNULL = open('/dev/null')

_echo_ = _stdout.write
_puke_ = _stderr.write

#from system.which import which

class Command(object):
	"""Command class"""
	sh_ = True
	su_ = False
	dbg = False
	timeout = None
	def __init__(self, *args, **kwargs):
		for arg in args:
			if hasattr(self, str(arg)):
				setattr(self, arg, True)
			elif hasattr(self, '%s_'%arg):
				setattr(self, '%s_'%arg, True)
		if self.dbg:
			print('\033[1;30m%s\033[0m'%str(Command.__mro__))
			print(' ', '\033[1;30m%s\033[0m'%str(self.__dict__))

	@staticmethod
	def __which(prog):
		"""which function like the linux 'which' program"""
		delim = ';' if osname == 'nt' else ':'
		__path = ''
		for path in environ['PATH'].split(delim):
			if access(pjoin(path, prog), X_OK):
				__path = pjoin(abspath(path), prog)
		return __path

	@staticmethod
	def _str(commands):
		"""list/tuple to str converter"""
		#print(commands)
		return ' '.join(str(command) for command in list(commands))

	@staticmethod
	def __sucmd(sudobin, commands):
		if 'sudo' in commands[0]:
			del commands[0]
		if int(getuid()) != 0:
			commands.insert(0, sudobin)
		return commands

	def _list(self, commands):
		"""
		commands string to list converter assuming at least one part
		"""
		#print(commands)
		for cmd in list(commands):
			if cmd and max(len(c) for c in cmd if c) == 1 and len(cmd) >= 1:
				return list(commands)
			return self._list(list(cmd))

	def _sudo(self, commands=None):
		"""privilege checking function"""
		sudo = self.__which('sudo')
		if not commands:
			if getuid() == 0: return True
			if int(run([sudo, '-v']).returncode) == 0:
				return True
			sucmds = None
		else:
			sucmds = self.__sucmd(sudo, commands)
		return sucmds

	def __cmdprep(self, commands, func):
		commands = self._list(commands)
		if self.su_:
			commands = self._sudo(commands)
		if self.sh_:
			commands = self._str(commands)
		if self.dbg:
			_echo_('\033[01;30m%s\n  `%s`\t{sh: %s, su: %s}\033[0m\n'%(
                func, commands, self.sh_, self.su_))
		return commands

	def run(self, *commands):
		"""just run the command and return the processes PID"""
		commands = self.__cmdprep(commands, self.run)
		return Popen(
            commands,
            stdout=DEVNULL, stderr=DEVNULL, shell=self.sh_).pid

	def call(self, *commands, stdout=True, stderr=True, inputs=None, b2s=None):
		"""
		default command execution
		prints STDERR, STDOUT and returns the exitcode
		"""
		inputs = inputs.encode() if (inputs and b2s) else inputs
		stderr = stderr if stderr else DEVNULL
		stdout = stdout if stdout else DEVNULL
		commands = self.__cmdprep(commands, self.call)
		return int(run(
            commands, shell=self.sh_, stdout=stdout,
            stderr=stderr, timeout=self.timeout, input=inputs).returncode)

	def stdx(self, *commands, inputs=None, b2s=True):
		"""command execution which returns STDERR and/or STDOUT"""
		commands = self.__cmdprep(commands, self.stdx)
		inputs = inputs.encode() if (inputs and b2s) else inputs
		prc = Popen(commands, stdout=PIPE, stderr=PIPE, shell=self.sh_)
		out, err = prc.communicate(timeout=self.timeout, input=inputs)
		if b2s and out:
			out = out.decode()
		if b2s and err:
			err = err.decode()
		return out, err

	def stdo(self, *commands, inputs=None, b2s=True):
		"""command execution which returns STDOUT only"""
		inputs = inputs.encode() if (inputs and b2s) else inputs
		commands = self.__cmdprep(commands, self.stdo)
		prc = Popen(commands, stdin=PIPE, stdout=PIPE, stderr=DEVNULL, shell=self.sh_)
		out, _ = prc.communicate(input=inputs, timeout=self.timeout)
		if b2s:
			out = out.decode()
		return out

	def stde(self, *commands, inputs=None, b2s=True):
		"""command execution which returns STDERR only"""
		inputs = inputs.encode() if (inputs and b2s) else inputs
		commands = self.__cmdprep(commands, self.stde)
		prc = Popen(commands, stdout=PIPE, stderr=PIPE, shell=self.sh_)
		_, err = prc.communicate(timeout=self.timeout, input=inputs)
		if b2s and err:
			err = err.decode()
		return err

	def erno(self, *commands, inputs=None, b2s=True):
		"""command execution which returns the exitcode only"""
		inputs = inputs.encode() if (inputs and b2s) else inputs
		commands = self.__cmdprep(commands, self.erno)
		prc = Popen(commands, stdout=DEVNULL, stderr=DEVNULL, shell=self.sh_)
		prc.communicate(timeout=self.timeout, input=inputs)
		return int(prc.returncode)

	def oerc(self, *commands, inputs=None, b2s=True):
		"""command execution which returns STDERR only"""
		inputs = inputs.encode() if (inputs and b2s) else inputs
		commands = self.__cmdprep(commands, self.oerc)
		prc = Popen(commands, stdout=PIPE, stderr=PIPE, shell=self.sh_)
		out, err = prc.communicate(timeout=self.timeout, input=inputs)
		if b2s and out:
			out = out.decode()
		if b2s and err:
			err = err.decode()
		return out, err, int(prc.returncode)


command = executor = Command('sh')
sudo = sucommand = Command('sh', 'su')

def sudofork(*args):
	"""sudo command fork wrapper function"""
	if not [i for i in getresuid() if i != 0]: return 1
	try:
		eno = int(sucommand.call(args))
	except KeyboardInterrupt:
		_echo_('\033[34maborted by keystroke\033[0m\n')
		eno = 1
	return eno

if __name__ == '__main__':
	exit(1)
